-- Flame

for i = 1, #mobjinfo-1 do
	if not ((mobjinfo[i].flags & MF_ENEMY)
	or (mobjinfo[i].flags & MF_BOSS)) then continue end
	
	-- Modify health on spawn
	addHook("MobjSpawn", function(mo)
		if not valid(mo) then return false end
		if (mo.flags & MF_BOSS) then
			mo.health = $*cv_mhbossmod.value
		elseif (mo.flags & MF_ENEMY) then
			mo.health = $*cv_mhenemymod.value
		end
	end, i)

	-- Don't modify existing behavior for objects with inherrently more than 1 health.
	--if (mobjinfo[i].spawnhealth > 1) then continue end 

	-- No painsound? Give it one!
	if not (mobjinfo[i].painsound) then
		mobjinfo[i].painsound = sfx_dmpain
	end
	
	-- No painstate? Give it one!
	if not (mobjinfo[i].painstate) then
		mobjinfo[i].painstate = mobjinfo[i].spawnstate
	end

	-- Mobj behavior while in damage
	addHook("MobjThinker", function(mo)
		if not valid(mo) then return false end
		if (mo.fuse >= 1) then mo.flags2 = $ ^^ (MF2_DONTDRAW) end
	end, i)
	
	-- What happens when the fuse runs out?
	addHook("MobjFuse", function(mo)
		if not valid(mo) then return false end
		if (mo.health >= 1) then -- More than one health?
			mo.fuse = 0
			mo.flags2 = $ & ~(MF2_FRET|MF2_DONTDRAW)
			return true -- Override behavior.
		end
	end, i)
	
	addHook("MobjDamage", function(target, inflictor, source, damage, damagetype)
		if not valid(target) then return false end
		if target.fuse then return false end -- Invincible

		-- Your current speed affects your damage! (If enabled!)
		if cv_mhspeedmod.value 
		and (valid(inflictor.player) or valid(source.player)) then
			local p = inflictor.player or source.player
			damage = max(1, (p.speed/10)>>FRACBITS)
		end
		
		-- Display how much damage you're doing
		if cv_mhdebug.value then
			print("Enemy Health: " .. target.health, "Damage dealt: " .. damage)
		end
		if ((target.health - damage) >= 1) then
			local ang = target.angle
			if valid(inflictor) then
				ang = R_PointToAngle2(inflictor.x, inflictor.y, target.x, target.y)
			elseif valid(source) then
				ang = R_PointToAngle2(source.x, source.y, target.x, target.y)
			end
			
			if (target.info.painsound) then S_StartSound(target, target.info.painsound) end
			target.fuse = TICRATE/2
			target.flags2 = $ | MF2_FRET
			
			if not (target.flags & MF_BOSS) then -- Don't adjust thrust behavior for 
				-- P_IsObjectOnGround shenanigans
				if (target.eflags & MFE_VERTICALFLIP) then
					target.z = $ - 1
				else
					target.z = $ + 1
				end
				
				P_SetObjectMomZ(target, FixedDiv(source.momz, FixedDiv(27*FRACUNIT,20*FRACUNIT)), true)
				
				if (P_MobjFlip(target)*target.momz < 5*FRACUNIT) then
					P_SetObjectMomZ(target, 5*FRACUNIT, false)
				end
				
				P_InstaThrust(target, ang, target.momz)
			end
			
			-- Bounce back!
			if valid(source.player) then
				local p = source.player
				if (p.pflags & PF_JUMPED) then
					if (target.flags & MF_BOSS) -- Bosses are strong!
					and valid(p.mo) then
						p.mo.momx = -$
						p.mo.momy = -$
						--P_InstaThrust(p.mo, -ang, p.speed)
					end
					
					if (P_MobjFlip(source)*source.momz < 0) then
						P_SetObjectMomZ(source, -source.momz, false)
					end
					
					if (P_MobjFlip(source)*source.momz < 5*FRACUNIT)
						P_SetObjectMomZ(source, FixedMul(5*FRACUNIT,27*FRACUNIT/20), false)
					end
				end
			end
			target.health = $ - damage
			return true
		else
			target.health = 0
			target.flags2 = $ & ~(MF2_DONTDRAW)
			target.flags = $ | MF_NOGRAVITY
			target.fuse = 0
			target.momx = 0
			target.momy = 0
			target.momz = 0
			return false
		end
	end, i)
end